#ifndef MIXSNET_ALGORITHMS_H
#define MIXSNET_ALGORITHMS_H

#include<ColoredWeightedGraph.hpp>
#include<Matrix.hpp>
#ifdef _OPENMP
    #include <omp.h>
#else
    #define omp_get_num_threads() 0
    #define omp_get_thread_num() 0
#endif

namespace mixsnet{

  template<typename result_type_t, class TGraph, class TMatrix, class TFunctor, class TColor>
  result_type_t forEachYij(TGraph& net, TMatrix& Y, TFunctor func, TColor& colors, bool directed = false)
  {
    typename TGraph::VertexIterator it = net.verticesBegin(), it2;
    typename TMatrix::iterator1 itmat1 = Y.begin1();
    unsigned int nbv = net.nbVertices();
    for (unsigned int i=0; i<nbv; i++){
        typename TMatrix::iterator2 itmat2 = itmat1.begin();
        it2 = net.verticesBegin();
        unsigned int last = i;
        if (directed) last = nbv;
        for (unsigned int j=0; j<last; j++){
            func(i, j, colors[net.getIndex(*it)], colors[net.getIndex(*it2)], *itmat2);
            it2++; itmat2++;
        }
        it++; itmat1++;
    }
    return func.exit();
  }

  template<typename result_type_t, class TGraph, class TMatrix, class TFunctor, class TColor>
  result_type_t forEachYidot(unsigned int i, TGraph& net, TMatrix& Y, TFunctor func, TColor& colors, bool directed = false)
  {
    typename TGraph::Vertex v = net.getVertexByIndex(i);
    typename TGraph::VertexIterator it2 = net.verticesBegin();
    typename TMatrix::iterator2 itmat2 = (Y.begin1()+i).begin();
    unsigned int nbv = net.nbVertices();
    for (unsigned int j=0; j<nbv; j++){
        func(i, j, colors[net.getIndex(v)], colors[net.getIndex(*it2)], *itmat2);
        if (directed && (i!=j)){
            func(j, i, colors[net.getIndex(*it2)], colors[net.getIndex(v)], Y(j,i));
        }
        it2++; itmat2++;
    }
    return func.exit();
  }

}
#endif
