#ifndef MIXSNET_FUNCTORS_H
#define MIXSNET_FUNCTORS_H
#include<ColoredWeightedGraph.hpp>
#include <cmath>
namespace mixsnet{

  template<typename data_type_t, class TDistribution>
  class CompleteLogLikelihoodFunctor
  {
  protected:
    double _l1;
    TDistribution& _dist;
  public:
    CompleteLogLikelihoodFunctor(TDistribution& dist) : _l1(0), _dist(dist) {}
    ~CompleteLogLikelihoodFunctor() {}
    void operator() (unsigned int i, unsigned j, color_type ci, color_type cj, data_type_t yij){
      _l1 += _dist.logdensity(i, j, ci, cj, yij);
    }
    double exit() {return _l1;}
  };

  template<typename weight_type_t, typename data_type_t>
  class SpatialSpreadingFunctor
  {
  private:
    std::pair<std::pair<unsigned int, unsigned int>,  weight_type_t> _dmax;
    color_type _c;
    ColoredRoadNetwork<weight_type_t>& _net;
  public:
    SpatialSpreadingFunctor(ColoredRoadNetwork<weight_type_t>& net, unsigned int cinterest)
    : _c(cinterest), _net(net) {_dmax.second = -std::numeric_limits<weight_type_t>::max();}
    ~SpatialSpreadingFunctor() {}
    void operator() (unsigned int i, unsigned j, color_type ci, color_type cj, data_type_t yij){
      if ((ci==_c)&&(cj==_c)){
          weight_type_t dij = _net.shortestPath(i,j);
          if (dij>_dmax.second){
              _dmax.first.first = i;
              _dmax.first.second = j;
              _dmax.second = dij;
          }
      }
    }
    std::pair<unsigned int, unsigned int> exit() {return _dmax.first;}
  };

  template<typename data_type_t, class TDistribution>
  class DrawFunctor
  {
  private:
    TDistribution& _dist;
  public:
    DrawFunctor(TDistribution& dist) : _dist(dist) {}
    ~DrawFunctor() {}
    void operator() (unsigned int i, unsigned j, color_type ci, color_type cj, data_type_t& yij){
      yij = _dist.draw(i, j, ci, cj);
      /*while(yij<0){
          std::cerr<<"Warning: negative value was drawn but corrected"<<std::endl;
          yij = _dist.draw(i, j, ci, cj);
      }*/
    }
    void exit() {}
  };

  class IsEqual
  {
  public:
    IsEqual() {}
    ~IsEqual() {}
    bool operator() (double a, double b){
      return fabs(a-b)<std::numeric_limits<double>::epsilon();
    }
  };

}
#endif
