#ifndef MIXSNET_MATRIX_HPP
#define MIXSNET_MATRIX_HPP

#include<functional>
#include<fstream>
#include<sstream>
#include<stdlib.h>
#include<boost/numeric/ublas/matrix.hpp>
namespace mixsnet{

    template<typename data_type_t>
      class Matrix : public boost::numeric::ublas::matrix<data_type_t>
    {
    public:
      Matrix()
        : boost::numeric::ublas::matrix<data_type_t>() {}
      Matrix(unsigned int nblines)
        : boost::numeric::ublas::matrix<data_type_t>(nblines, nblines, data_type_t()) {}
      virtual ~Matrix(){}
      Matrix(const Matrix& m)
        : boost::numeric::ublas::matrix<data_type_t>(m) {}
      void read(const std::string& datafile, std::function<unsigned int (const std::string& name)> convertToIndex){
        std::vector<unsigned int> index;
        std::ifstream matrixstream;
        matrixstream.open(datafile.c_str());
        if (!matrixstream.good()){
            std::cerr<<"Error: unable to open file "<<datafile<<std::endl;
            exit(1);
        }
        std::string line;
        // first line
        getline(matrixstream, line);
        std::istringstream linestream(line);
        std::string id;
        linestream>>id;
        while (id.size()){
            unsigned int i = convertToIndex(id);
            index.push_back(i);
            id.clear();
            linestream>>id;
        }
        // next lines
        unsigned int nblines = index.size();
        this->resize(nblines, nblines);
        for (unsigned int i=0; i<nblines; i++){
            getline(matrixstream, line);
            std::istringstream mlinestream(line);
            unsigned int ii = index[i];
            for (unsigned int j=0; j<nblines; j++){
                unsigned int ij = index[j];
                data_type_t v;
                mlinestream>>v;
                (*this)(ii,ij) = v;
            }
        }
        matrixstream.close();
      }
      void write(const std::string& datafile, std::function<std::string (unsigned int i)> convertToName){
        std::ofstream matrixstream;
        matrixstream.open(datafile.c_str());
        if (!matrixstream.good()){
           std::cerr<<"Error: unable to open file "<<datafile<<std::endl;
            exit(1);
        }
        // first line
        for (unsigned int i=0; i<this->size1(); i++)
          matrixstream<<convertToName(i)<<" ";
        matrixstream<<std::endl;
        // next lines
        for (unsigned int i=0; i<this->size1(); i++){
            for (unsigned int j=0; j<this->size2(); j++)
              matrixstream<<(*this)(i,j)<<" ";
            matrixstream<<std::endl;
        }
        matrixstream.close();
      }
    };
}
#endif
